@@ -22,15 +22,16 @@
  * http://www.FutureQuest.net/
  * ossi@FutureQuest.net
  */
+#include <bglibs/sysdeps.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <sysdeps.h>
+#include <sys/stat.h>
 #include <cvm/sasl.h>
 #include <cvm/v2client.h>
-#include <iobuf/iobuf.h>
-#include <str/str.h>
+#include <bglibs/iobuf.h>
+#include <bglibs/str.h>
 
 const char program[] = "imapfront-auth";
 const int msg_show_pid = 1;
@@ -54,6 +55,9 @@ static str cmd;
 static str line_args[MAX_ARGC];
 static int line_argc;
 
+static int auth_count;
+static int auth_max;
+
 static struct sasl_auth saslauth = { .prefix = "+ " };
 
 void log_start(const char* tagstr)
@@ -245,15 +249,48 @@ void cmd_capability(void)
   respond(0, "OK CAPABILITY completed");
 }
 
+static int setup_env(void)
+{
+  const char* s;
+  const char* colon;
+  struct stat st;
+
+  if (cvm_fact_mailbox != 0
+      && (s = getenv("SETUP_ENV")) != 0
+      && strcmp(s, "dovecot") == 0) {
+    /* This tells Dovecot that its environment has already been set up. */
+    if (putenv("DOVECONF_ENV=1") != 0)
+      return 0;
+    /* Use the file type to set the prefix to mbox: or maildir:
+     * Assume missing files are mboxes. */
+    s = (stat(cvm_fact_mailbox, &st) == 0
+	 && S_ISDIR(st.st_mode))
+      ? "maildir:"
+      : "mbox:";
+    /* Use cmd for temporary storage of the substituted mailbox name */
+    if (!str_copys(&cmd, s))
+      return 0;
+    s = cvm_fact_mailbox;
+    while ((colon = strchr(s, ':')) != 0) {
+      if (!str_catb(&cmd, s, colon-s)
+	  || !str_catb(&cmd, "::", 2))
+	return 0;
+      s = colon + 1;
+    }
+    if (!str_cats(&cmd, s))
+      return 0;
+    cvm_fact_mailbox = cmd.s;
+  }
+  return cvm_setenv()
+    && setenv("IMAPLOGINTAG", tag.s, 1) == 0
+    && setenv("AUTHENTICATED", cvm_fact_username, 1) == 0;
+}
+
 void do_exec(void)
 {
   if (!cvm_setugid())
     respond(0, "NO Internal error: could not set UID/GID");
-  else if (!cvm_setenv() ||
-	   (cvm_fact_mailbox != 0 &&
-	    setenv("MAILDIR", cvm_fact_mailbox, 1) == -1) ||
-	   setenv("IMAPLOGINTAG", tag.s, 1) == -1 ||
-	   setenv("AUTHENTICATED", cvm_fact_username, 1) == -1)
+  else if (!setup_env())
     respond(0, "NO Internal error: could not set environment");
   else {
     alarm(0);
@@ -274,6 +311,9 @@ void cmd_login(int argc, str* argv)
       do_exec();
     else
       respond(0, "NO LOGIN failed");
+    ++auth_count;
+    if (auth_max > 0 && auth_count >= auth_max)
+      exit(0);
   }
 }
 
@@ -294,6 +334,9 @@ void cmd_authenticate(int argc, str* argv)
   respond_str("NO AUTHENTICATE failed: ");
   respond_str(sasl_auth_msg(&i));
   respond_end();
+    ++auth_count;
+    if (auth_max > 0 && auth_count >= auth_max)
+      exit(0);
 }
 
 struct command
@@ -338,10 +381,13 @@ static void dispatch_line(void)
 
 static int startup(int argc, char* argv[])
 {
+  const char* tmp;
   if (argc < 2) {
     respond(NOTAG, "NO Usage: imapfront-auth imapd [args ...]");
     return 0;
   }
+  if ((tmp = getenv("MAXAUTHFAIL")) != 0)
+    auth_max = strtoul(tmp, 0, 10);
   if ((domain = cvm_ucspi_domain()) == 0)
     domain = "unknown";
   nextcmd = argv + 1;
