@@ -1,6 +1,8 @@
 #include "responses.h"
 
-RESPONSE(accepted, 250, "2.6.0 Message accepted.");
+RESPONSE(accepted_message, 250, "2.6.0 Message accepted.");
+RESPONSE(accepted_sender, 250, "2.1.0 Sender accepted.");
+RESPONSE(accepted_recip, 250, "2.1.5 Recipient accepted.");
 RESPONSE(internal, 451, "4.3.0 Internal error.");
 RESPONSE(oom, 451, "4.3.0 Out of memory.");
 
